#!/usr/bin/env python

"A WebStack application for a system configurator."

import WebStack.Generic
import XSLForms.Fields
import XSLForms.Output
import XSLForms.Prepare
import XSLForms.Utils
import XSLOutput
import libxml2dom
import os

class ConfiguratorResource:

    "A resource providing a system configurator."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    encoding = "utf-8"
    in_page_resources = {
        "/cpu" : ("config_output_cpu.xsl", "cpu-node"),
        "/memory" : ("config_output_memory.xsl", "memory-node"),
        "/hard-disks" : ("config_output_harddisks.xsl", "hard-disks-node")
        }

    def get_fields_from_body(self, trans, encoding):
        text = trans.get_request_stream().read()
        parameters = {}
        for text_line in text.split("\r\n"):
            text_parts = text_line.split("=")
            text_name, text_value = text_parts[0], "=".join(text_parts[1:])
            if not parameters.has_key(text_name):
                parameters[text_name] = []
            # NOTE: Workaround from posted text.
            if text_value[-1] == "\x00":
                text_value = text_value[:-1]
            parameters[text_name].append(text_value)
        return parameters

    def respond(self, trans):

        global XSLForms # NOTE: Strangely required to avoid UnboundLocalError!

        # Only obtain field information according to the stated method.

        method = trans.get_request_method()
        path_info = trans.get_path_info()

        if method == "GET":
            fields = XSLForms.Fields.Fields(encoding="iso-8859-1", values_are_lists=1)
            parameters = trans.get_fields_from_path()
            documents = fields.make_documents(parameters.items())
        elif method == "POST":
            fields = XSLForms.Fields.Fields(encoding=self.encoding, values_are_lists=1)

            # Handle requests for fragments.

            if path_info in self.in_page_resources.keys():
                parameters = self.get_fields_from_body(trans, self.encoding)
            else:
                parameters = trans.get_fields_from_body(self.encoding)

            # Get the XML representation of the request.

            documents = fields.make_documents(parameters.items())
        else:
            trans.set_response_code(405)
            raise WebStack.Generic.EndOfResponse

        # Ensure the presence of a document.

        if documents.has_key("configuration"):
            configuration = documents["configuration"]
        else:
            configuration = fields.new_instance("configuration")
        #print "*", configuration.toString()

        # Add and remove elements according to the selectors found.

        selectors = fields.get_selectors(parameters.items(), documents)
        XSLForms.Utils.add_elements(selectors.get("add-memory-unit"), "memory-unit")
        XSLForms.Utils.remove_elements(selectors.get("remove-memory-unit"))
        XSLForms.Utils.add_elements(selectors.get("add-storage-unit"), "storage-unit")
        XSLForms.Utils.remove_elements(selectors.get("remove-storage-unit"))
        XSLForms.Utils.add_elements(selectors.get("add-hard-disk"), "hard-disk")
        XSLForms.Utils.remove_elements(selectors.get("remove-hard-disk"))

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", self.encoding))
            configuration.toStream(trans.get_response_stream(), trans.get_response_stream_encoding())

        # When not exported, the data is transformed to produce a normal Web
        # page.

        else:

            # Transform, adding enumerations/ranges.

            database_xsl = os.path.join(self.resource_dir, "config_database.xsl")
            database_xml = os.path.join(self.resource_dir, "config_database.xml")
            proc = XSLOutput.Processor([database_xsl], references={"database" : database_xml})
            configuration = proc.get_result(configuration)

            # Start the response.

            trans.set_content_type(WebStack.Generic.ContentType("application/xhtml+xml", self.encoding))

            # Define the stylesheet parameters.

            stylesheet_parameters = {}

            # Ensure that an output stylesheet exists.

            if path_info in self.in_page_resources.keys():
                trans_xsl_filename, trans_xsl_region = self.in_page_resources[path_info]
                template_xml = os.path.join(self.resource_dir, "config_output.xsl")
                trans_xsl = os.path.join(self.resource_dir, trans_xsl_filename)
                XSLForms.Prepare.ensure_stylesheet_fragment(template_xml, trans_xsl, trans_xsl_region)
                element_path = parameters.get("element-path", [""])[0]
                stylesheet_parameters["element-path"] = element_path
                #print "*", repr(stylesheet_parameters["element-path"])
            else:
                trans_xsl = os.path.join(self.resource_dir, "config_output.xsl")
                template_xml = os.path.join(self.resource_dir, "config_template.xhtml")
                XSLForms.Prepare.ensure_stylesheet(template_xml, trans_xsl)

            stylesheet_parameters["application-url"] = \
                "http://%s:%s%s" % (trans.get_server_name(), trans.get_server_port(), trans.get_path_without_query())

            # Complete the response.

            proc = XSLOutput.Processor([trans_xsl], parameters=stylesheet_parameters)
            proc.send_output(trans.get_response_stream(), trans.get_response_stream_encoding(),
                configuration)

            #import sys
            #proc = XSLOutput.Processor([trans_xsl], parameters=stylesheet_parameters)
            #proc.send_output(sys.stderr, "iso-8859-1", configuration)

# vim: tabstop=4 expandtab shiftwidth=4
